@@ -1,7 +1,7 @@
-package Date::Baha::i;
+package Date::Baha::i;  # {{{
 
 use strict;
-use vars qw($VERSION); $VERSION = '0.08.1';
+use vars qw($VERSION); $VERSION = '0.11.1';
 use base qw(Exporter);
 use vars qw(@EXPORT @EXPORT_OK);
 @EXPORT = @EXPORT_OK = qw(
@@ -10,7 +10,6 @@ use vars qw(@EXPORT @EXPORT_OK);
     date
     days
     days_of_the_week
-    greg_to_bahai
     holy_days
     months
     next_holy_day
@@ -20,13 +19,13 @@ use vars qw(@EXPORT @EXPORT_OK);
 use Date::Calc qw(
     leap_year
     Add_Delta_Days
-    Date_to_Time
     Day_of_Week
     Delta_Days
     Timezone
 );
 use Lingua::EN::Numbers::Ordinate;
 use Lingua::Num2Word;
+# }}}
 
 # Set constants {{{
 use constant FACTOR => 19;
@@ -78,7 +77,7 @@ use constant MONTH_DAY => qw(
     Sultan
     Mulk
     'Ala
-    Ayyam-i-ha
+    Ayyam-i-Ha
 );
 use constant DOW_NAME => qw(
     Jalal
@@ -92,8 +91,10 @@ use constant DOW_NAME => qw(
 use constant HOLY_DAYS => {
     # Work suspended:
     "Naw Ruz"                   => [ 3, 21],
-    "Ridvan"                    => [ 4, 21, 12],
+    "Days of Ridvan"            => [ 4, 21, 12],
+    "First Day of Ridvan"       => [ 4, 21],
     "Ninth Day of Ridvan"       => [ 4, 29],
+    "Twelfth Day of Ridvan"     => [ 5,  2],
     "Declaration of the Bab"    => [ 5, 23],
     "Ascension of Baha'u'llah"  => [ 5, 29],
     "Martyrdom of the Bab"      => [ 7,  9],
@@ -116,34 +117,31 @@ sub days_of_the_week { return DOW_NAME }
 sub holy_days { return HOLY_DAYS }
 # }}}
 
-# date function {{{
-sub date {
+sub date {  # {{{
     my %args = @_;
 
-    my ($year, $month, $day);
+    my ($year, $month, $day) = @args{qw(year month day)};
 
-    if ($args{use_gmtime}) {
-        ($year, $month, $day) = $args{timestamp}
-            ? (gmtime($args{timestamp}))[5,4,3]
-            : (gmtime)[5,4,3];
-    }
-    else {
-        ($year, $month, $day) = $args{timestamp}
-            ? (localtime($args{timestamp}))[5,4,3]
-            : (localtime)[5,4,3];
+    unless ($year && $month && $day) {
+        if ($args{use_gmtime}) {
+            ($year, $month, $day) = $args{timestamp}
+                ? (gmtime($args{timestamp}))[5,4,3]
+                : (gmtime)[5,4,3];
+        }
+        else {
+            ($year, $month, $day) = $args{timestamp}
+                ? (localtime($args{timestamp}))[5,4,3]
+                : (localtime)[5,4,3];
+        }
+
+        # Fix the year and the month.
+        $year += ADJUST_YEAR;
+        $month++;
     }
 
     # This is what will eventually be used in the return.
     my ($bahai_month, $bahai_day);
 
-    # Fix the year and the month.
-    $year += ADJUST_YEAR;
-    $month++;
-
-    # XXX Argh!  Why do I need to do this?  Am I deluded?
-    ($year, $month, $day) = Add_Delta_Days($year, $month, $day, 1)
-        unless $args{use_gmtime};
-
     # Begin with the first month of the year (at the Spring equinox).
     my ($m, $d) = (MARCH, YEAR_START_DAY);
 
@@ -197,19 +195,9 @@ sub date {
 }
 # }}}
 
-# greg_to_bahai function {{{
-sub greg_to_bahai {
-    my ($y, $m, $d, %args) = @_;
-    return date (
-        timestamp => Date_to_Time ($y, $m, $d, 0, 0, 0),
-        %args
-    );
-}
-# }}}
-
-# as_string function {{{
-# XXX With Lingua::Num2Word, naively assume that we only care about English.
-sub as_string {
+sub as_string {  # {{{
+    # XXX With Lingua::Num2Word, naively assume that we only care
+    # about English.
     my ($date_hash, %args) = @_;
 
     $args{size}     = 1 unless defined $args{size};
@@ -298,13 +286,11 @@ sub as_string {
     }
 
     return $date;
-}
-# }}}
+}  # }}}
 
-# next_holy_day function {{{
-# NOTE: We are only concerned with the month/day, so the date year
-# is ignored.
-sub next_holy_day {
+sub next_holy_day {  # {{{
+    # NOTE: We are only concerned with the month/day, so the date year
+    # is ignored.
     my ($year, $month, $day) = @_;
 
     # Make the month and day a pseudo real number.
@@ -328,8 +314,7 @@ sub next_holy_day {
     $holy_date = $sorted[-1] unless $holy_date;
 
     return { $inverted{$holy_date} => [ split /\./, $holy_date ] };
-}
-# }}}
+}  # }}}
 
 # Helper functions {{{
 # The Baha'i week starts on Saturday.
@@ -441,10 +426,10 @@ sub _invert_holy_days {
 
         # Does this date contain a day span?
         if (@$dates == 3) {
-            # Increment the Ayyam-i-ha day if we are in a leap year.
-            $dates->[2]++ if leap_year ($year);
+            # Increment the Ayyam-i-Ha day if we are in a leap year.
+            $dates->[2]++ if $name eq 'Ayyam-i-ha' && leap_year ($year);
 
-            for (1 .. $dates->[2]) {
+            for (1 .. $dates->[2] - 1) {
                 # Pre-pad the day number with a zero.
                 (undef, my $month, my $day) = Add_Delta_Days($year, @$dates[0,1], $_);
                 $day = sprintf '%02d', $day;
@@ -469,10 +454,12 @@ Date::Baha::i - Compute the numeric and named Baha'i date.
   use Date::Baha'i;
 
   %bahai_date = date ();
-  $bahai_date = date ();
-
-  %bahai_date = greg_to_bahai ($year, $month, $day);
-  $bahai_date = greg_to_bahai ($year, $month, $day);
+  %bahai_date = date (timestamp => $secs_since_1970);
+  $bahai_date = date (
+      year  => $year,
+      month => $month,
+      day   => $day,
+  );
 
   $bahai_date = as_string (\%bahai_date);
 
@@ -632,7 +619,8 @@ Commemorates the date in 1819 when the Bab was born in Shiraz, Iran
 
 * Birth of Baha'u'llah - 12 November
 
-Commemorates the date in 1817 when Baha'u'llah was born in Tihran, Iran
+Commemorates the date in 1817 when Baha'u'llah was born in Tihran, 
+Iran
 
 - Work does not have to cease on these Holy Days:
 
@@ -707,16 +695,30 @@ L<http://www.moonwise.co.uk/year/159bahai.htm>
 
 =head2 date
 
+  %bahai_date = date ();
   %bahai_date = date (
-      timestamp => $timestamp,
+      timestamp  => $secs_since_1970,
       use_gmtime => $use_gmtime,
       %args,
-  )
+  );
+
+  $bahai_date = date (
+      year  => $year,
+      month => $month,
+      day   => $day,
+      %args,
+  );
 
 This function returns a hash of the Baha'i date names and numbers from
-a system or user provided time () stamp.  Also, this function can be
-forced to use gmtime instead of localtime (the default).  The extra 
-arguments are used for the as_string () function, detailed below.
+either epoch seconds or a year, month, day triple.
+
+If using epoch seconds (timestamp), this function can be forced to
+use gmtime instead of localtime.  If neither a timestamp or ymd 
+triple are given, the system localtime (or gmtime) are used as a 
+default.
+
+The extra arguments are used for the as_string () function, which are 
+detailed below.
 
 In a scalar context, this function returns a string sentence with the 
 numeric and named Baha'i date.  In an array context, it returns a hash
@@ -737,23 +739,6 @@ with the following keys:
   timezone
   holy_day
 
-=head2 greg_to_bahai
-
-  %bahai_date = greg_to_bahai (
-      $year, $month, $day,
-      use_gmtime => $use_gmtime,
-      %args,
-  );
-
-Compute the Baha'i date from a Gregorian year, month, day triple.
-Also, this function can be forced to use gmtime instead of localtime 
-(the default).  The extra arguments are used for the as_string () 
-function, detailed below.
-
-In a scalar context, this function returns a string sentence with the 
-numeric and named Baha'i date.  In an array context, it returns the 
-date () hash (described above).
-
 =head2 as_string
 
   $date = as_string (
@@ -782,7 +767,7 @@ alphanumeric representations on or off.  The defaults are as follows:
 Thus, "long non-numeric alpha without the timezone" is the default 
 representation.
 
-Here are some handy examples:
+Here are some handy examples (newlines added for readability):
 
   short numeric:
   7, 1/1/159
@@ -791,10 +776,13 @@ Here are some handy examples:
   7, 1/1/159, -6
 
   long numeric:
-  7th day of the week, 1st day of the 1st month, year 159, 7th year of the 9th vahid of the 1st kull-i-shay, holy day: Naw Ruz
+  7th day of the week, 1st day of the 1st month, year 159,
+  7th year of the 9th vahid of the 1st kull-i-shay, holy day: Naw Ruz
 
   long numeric with TZ:
-  7th day of the week, 1st day of the 1st month, year 159, 7th year of the 9th vahid of the 1st kull-i-shay, TZ -6h, holy day: Naw Ruz
+  7th day of the week, 1st day of the 1st month, year 159,
+  7th year of the 9th vahid of the 1st kull-i-shay, TZ -6h,
+  holy day: Naw Ruz
 
   short alpha:
   Istiqlal, Baha of Baha, Abad of Baha
@@ -803,28 +791,40 @@ Here are some handy examples:
   Istiqlal, Baha of Baha, Abad of Baha, TZ -6h
 
   long alpha:
-  week day Istiqlal, day Baha of month Baha, year one hundred fifty nine of year Abad of the vahid Baha of the 1st kull-i-shay, holy day: Naw Ruz
+  week day Istiqlal, day Baha of month Baha,
+  year one hundred fifty nine of year Abad of the vahid Baha of the
+  1st kull-i-shay, holy day: Naw Ruz
 
   long alpha with TZ:
-  week day Istiqlal, day Baha of month Baha, year one hundred fifty nine of year Abad of the vahid Baha of the 1st kull-i-shay, with timezone offset of negative six hours, holy day: Naw Ruz
+  week day Istiqlal, day Baha of month Baha,
+  year one hundred fifty nine of year Abad of the vahid Baha of the
+  1st kull-i-shay, with timezone offset of negative six hours,
+  holy day: Naw Ruz
 
   short alpha-numeric:
   Istiqlal (7), Baha (1) of Baha (1), year 159, Abad (7) of Baha (9)
 
   short alpha-numeric with TZ:
-  Istiqlal (7), Baha (1) of Baha (1), year 159, Abad (7) of Baha (9), TZ -6h
+  Istiqlal (7), Baha (1) of Baha (1), year 159, Abad (7) of Baha (9),
+  TZ -6h
 
   long alpha-numeric:
-  7th week day Istiqlal, 1st day Baha of the 1st month Baha, year one hundred fifty nine (159), 7th year Abad of the 9th vahid Baha of the 1st kull-i-shay, holy day: Naw Ruz
+  7th week day Istiqlal, 1st day Baha of the 1st month Baha,
+  year one hundred fifty nine (159), 7th year Abad of the
+  9th vahid Baha of the 1st kull-i-shay, holy day: Naw Ruz
 
   long alpha-numeric with TZ:
-  7th week day Istiqlal, 1st day Baha of the 1st month Baha, year one hundred fifty nine (159), 7th year Abad of the 9th vahid Baha of the 1st kull-i-shay, with timezone offset of negative six hours, holy day: Naw Ruz
+  7th week day Istiqlal, 1st day Baha of the 1st month Baha,
+  year one hundred fifty nine (159), 7th year Abad of the
+  9th vahid Baha of the 1st kull-i-shay,
+  with timezone offset of negative six hours, holy day: Naw Ruz
 
 =head2 next_holy_day
 
-  %holy_day = next_holy_day ($year, $month, $day)
+  %holy_day = next_holy_day ($year, $month, $day);
 
-This function returns the first holy day after the provided date triple.
+This function returns the first holy day after the provided date
+triple.
 
 The return is a hash reference with a single key (the name of the
 holy day) and an two element array reference of [month, day] as the 
@@ -881,22 +881,24 @@ L<Lingua::Num2Word>
 
 =head1 TO DO
 
-Convert to Gregorian dates and Unix time stamps from Baha'i dates.
+Convert to standard dates and Unix time stamps from Baha'i dates.
 
-Integrate with the DateTime namespace functionality.
-
-Output unicode.
+Optionally output unicode.
 
 Base the date computation on the time of day (the Baha'i day begins at 
 Sunset) - and the location longitude/latitude.  Yes, that would be 
 with Astro::Sunrise::sunset ().
 
-Overload localtime () and gmtime () just to be cool?
+Overload localtime and gmtime, just to be cool?
 
 =head1 DEDICATION
 
 Hi Kirsten  : )
 
+=head1 REFERENCES
+
+L<http://www.perl.com/pub/a/2003/03/13/datetime.html>
+
 =head1 AUTHOR
 
 Gene Boggs E<lt>cpan@ology.netE<gt>
